package com.fossgalaxy.games.ggj2017;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.Map;

import com.fossgalaxy.object.annotations.ObjectDef;
import org.codetome.hexameter.core.api.CubeCoordinate;
import org.codetome.hexameter.core.api.Hexagon;

import com.fossgalaxy.games.rts.GameState;
import com.fossgalaxy.games.rts.entity.Entity;
import com.fossgalaxy.games.rts.entity.EntityType;
import com.fossgalaxy.games.rts.entity.HexagonTile;
import com.fossgalaxy.games.rts.entity.ResourceType;
import com.fossgalaxy.games.rts.io.SpriteRegistry;
import com.fossgalaxy.games.rts.order.BuildOrder;
import com.fossgalaxy.games.rts.order.Order;
import com.fossgalaxy.games.rts.ui.GameView;

public class BuildAction extends DroneAction {
	private final Color HINT_BUILD_COLOUR = new Color(243, 163, 32, 50);
	private final Stroke HINT_STROKE = new BasicStroke(2f);
	private EntityType type;

	@ObjectDef("BuildAction")
	public BuildAction(EntityType type) {
		this.type = type;
	}
	
	@Override
	public void renderHints(Graphics2D g, GameState s, Entity actor) {
		if (actor == null) {
			return;
		}
		
		if (!JamApp.isZoneOfControl(s, actor)) {
			SpriteRegistry sr = SpriteRegistry.INSTANCE;
			BufferedImage img = sr.getImage("ui/no-signal");
			
			Hexagon<HexagonTile> ht = s.cube2hex(actor.getPos());
			
			g.drawImage(img, (int)ht.getCenterX() - 25, (int)ht.getCenterY() - 25, 50, 50, null);
			
			return;
		}
		
		Collection<Hexagon<HexagonTile>> range = s.getRange(actor.getPos(), 1);
		
		Area area = new Area();
		
		range.forEach(h -> {
	        if (!isPossible(actor, s, h.getCubeCoordinate())) {
	        	return;
	        }
	        
		    Shape hexShape = GameView.hex2shape.apply(h);
	        area.add(new Area(hexShape));
	        
	        g.setColor(getHintColour());
	        g.fill(hexShape);
		});
		
        g.setColor(getBorderColour());
        g.setStroke(HINT_STROKE);
        g.draw(area);
		
	}

	@Override
	public Order generateOrder(CubeCoordinate co, GameState s) {
		return new BuildOrder(type, co);
	}
	
	@Override
	public boolean isPossible(Entity entity, GameState s, CubeCoordinate co) {
		if (!super.isPossible(entity, s, co)) {
			return false;
		}
		
		//is the tile free?
		Entity entityAt = s.getEntityAt(co);
		if (entityAt != null) {
			return false;
		}
		
		//can we afford it?
		for (Map.Entry<ResourceType, Integer> costEntry : type.getCosts().entrySet()) {	
			int currVal = s.getResource(entity.getOwner(), costEntry.getKey());
			if (currVal < costEntry.getValue()) {
				return false;
			}	
		}
		
		//are we next to the space?
		int distance = s.getDistance(entity.getPos(), co);
		if (distance != 1) {
			return false;
		}
		
		return true;
	}

	@Override
	public boolean isPossible(Entity entity) {
		return true;
	}

	@Override
	public Color getHintColour() {
		return HINT_BUILD_COLOUR;
	}

	@Override
	public Color getBorderColour() {
		return Color.YELLOW;
	}

	public String toString() {
		return "build "+type.getName();
	}
	
}
